/*
 *  stereo -- Stereo matching software
 *  $Id: parameters.c 204 2004-07-27 11:53:39Z sam $
 *  Copyright (c) 2004 Sam Hocevar <sam@zoy.org>
 *                     Éric Beets <ericbeets@free.fr>
 *  All Rights Reserved
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA
 */

#include <stdint.h>
#include <stdio.h>
#include <math.h>

#include "defs.h"
#include "main.h"
#include "image.h"
#include "geometry.h"

static int compute_points(image_t *image,
                          projection_t *data);

static pt2d_t transform(projection_t *data,
                        pt2d_t from);

static inline double dsq(double x)
{
    return x * x;
}

/* Precalculate transformation parameters */
int parameters(image_t *image,
               projection_t *data)
{
    int i;

    /* Compute O, I, Os ... */
    data->d = sqrt(image->width * image->width + image->height * image->height);
    data->D = sqrt(image->width * image->width + image->height * image->height);

    compute_points(image, data);

    for(i = 100; i--;)
    {
        pt2d_t p1, p2;

        p1 = data->scanline[data->scanline_count/4].a;
        p1 = transform(data, p1);
        p2 = data->scanline[data->scanline_count*3/4].a;
        p2 = transform(data, p2);

        if(abs(p2.y - p1.y) < image->height * 0.999)
            data->d *= 0.999;
        else if(abs(p2.y - p1.y) > image->height * 1.001)
            data->d *= 1.001;
        else break;

        compute_points(image, data);
    }

    printf("point I: (%g, %g, %g)\n", data->I.x, data->I.y, data->I.z);
    printf("point O: (%g, %g, %g)\n", data->O.x, data->O.y, data->O.z);
    printf("point Os: (%g, %g, %g)\n", data->Os.x, data->Os.y, data->Os.z);
    printf("vx: (%g, %g, %g)\n", data->vx.x, data->vx.y, data->vx.z);
    printf("vy: (%g, %g, %g)\n", data->vy.x, data->vy.y, data->vy.z);

    return 0;
}

static int compute_points(image_t *image,
                          projection_t *data)
{
    double tmp, n;
    pt3d_t vx, vy, vz;

    data->I.x = data->intersect.x;
    data->I.y = data->intersect.y;
    data->I.z = data->d;

    data->O.x = 0.0;
    data->O.y = 0.0;
    data->O.z = data->d;

    tmp = sqrt(dsq(data->d * data->I.x) + dsq(data->d * data->I.y)
                + dsq(dsq(data->I.x) + dsq(data->I.y)));
    data->Os.x = - data->d * data->D * data->I.x / tmp;
    data->Os.y = - data->d * data->D * data->I.y / tmp;
    data->Os.z = data->D * (dsq(data->I.x) + dsq(data->I.y)) / tmp;

    /* Compute vx, vy and vz */
    vx = data->I;
    n = norm(vx);
    vx = multiply(1.0 / n, vx);

    if(vx.x < 0)
        vx = multiply(-1.0, vx);

    vy = vector_product(data->Os, vx);
    n = norm(vy);
    vy = multiply(1.0 / n, vy);

    if(vy.y < 0)
        vy = multiply(-1.0, vy);

    vz = vector_product(vx, vy);

    data->vx = vx;
    data->vy = vy;
    data->vz = vz;

    return 0;
}

static pt2d_t transform(projection_t *data,
                        pt2d_t from)
{
    pt3d_t pt3d;
    pt2d_t to;

    pt3d.x = from.x;
    pt3d.y = from.y;
    pt3d.z = data->d;

    pt3d = multiply(1 / scalar_product(pt3d, data->Os), pt3d);
    pt3d = multiply(scalar_product(data->Os, data->Os), pt3d);

    to.x = scalar_product(pt3d, data->vx);
    to.y = scalar_product(pt3d, data->vy);

    return to;
}

